// Copyright 2015 The Go Authors. All rights reserved.
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file.

// package sys contains system- and configuration- and architecture-specific
// constants used by the runtime.
package sys
