// Copyright 2021 The Go Authors. All rights reserved.
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file.

package main

import "./a"

// Testing inlining of functions that refer to instantiated exported and non-exported
// generic types.

func main() {
	a.Test1()
	a.Test2()
	a.Test3()
}
